
/* A LiPo storage discharger by Axel Brinkeby 2014-10-28
    www.axelsdiy.brinkeby.se   
*/

const int loadSwitch = 2;
const int doneLED = 3;
const int voltageReadPin = 0;

const int LED1pin = 4;
const int LED2pin = 5;
const int LED3pin = 6;
const int LED4pin = 7;

const float TARGET_VOLTAGE = 11.55;   // 3.85 volt per cell
const float NUMBER_OF_SAMPLES = 100; 

float startVoltage = 0; 
float LED1voltageThreshold = 0;
float LED2voltageThreshold = 0;
float LED3voltageThreshold = 0;
float LED4voltageThreshold = 0;


void setup() {
  pinMode(loadSwitch, OUTPUT);
  digitalWrite(loadSwitch, LOW);
  pinMode(doneLED, OUTPUT);
  digitalWrite(doneLED, LOW);
  pinMode(LED1pin, OUTPUT);
  digitalWrite(LED1pin, LOW);
  pinMode(LED2pin, OUTPUT);
  digitalWrite(LED2pin, LOW);
  pinMode(LED3pin, OUTPUT);
  digitalWrite(LED3pin, LOW);
  pinMode(LED4pin, OUTPUT);
  digitalWrite(LED4pin, LOW);

  Serial.begin(115200);

  Serial.println("======");
  
  digitalWrite(loadSwitch, HIGH);
  Serial.println("laod switched ON, discharge started");
  
  Serial.println("Waiting 1 second for voltage to stabilize...");
  delay(1000);

  startVoltage = getVoltage(); 
  Serial.print("Battery start voltage: ");
  Serial.println(startVoltage,3);
  
  Serial.println("Calculating progres LED Threshold values...");
  float voltageDiff = getVoltage() - TARGET_VOLTAGE;
  
  LED1voltageThreshold = TARGET_VOLTAGE; 
  Serial.print("LED1voltageThreshold: ");
  Serial.println(LED1voltageThreshold,3);
  
  LED2voltageThreshold = TARGET_VOLTAGE + (voltageDiff / 4); 
  Serial.print("LED2voltageThreshold: ");
  Serial.println(LED2voltageThreshold,3);
  
  LED3voltageThreshold = TARGET_VOLTAGE + 2 * (voltageDiff / 4); 
  Serial.print("LED3voltageThreshold: ");
  Serial.println(LED3voltageThreshold,3);
  
  LED4voltageThreshold = TARGET_VOLTAGE + 3 * (voltageDiff / 4);
  Serial.print("LED4voltageThreshold: ");
  Serial.println(LED4voltageThreshold,3);   
}

void loop()
{
  if (!digitalRead(doneLED)) {    
    float voltage = getVoltage(); 
    Serial.print("Battery voltage: ");
    Serial.println(voltage,3);
    
    // update progress LEDs
    digitalWrite(LED1pin, (voltage > LED1voltageThreshold)); 
    digitalWrite(LED2pin, (voltage > LED2voltageThreshold)); 
    digitalWrite(LED3pin, (voltage > LED3voltageThreshold)); 
    digitalWrite(LED4pin, (voltage > LED4voltageThreshold)); 
     
    if (voltage < TARGET_VOLTAGE) {
      digitalWrite(loadSwitch, LOW);
      Serial.println("Laod switched OFF, discharge completed");
      digitalWrite(doneLED, HIGH);
    }    
  }  
  delay(1000);
}

float getVoltage() {
  float voltageSum = 0;   
  for (int i = 0; i < NUMBER_OF_SAMPLES; i++) 
  {    
    voltageSum += analogRead(voltageReadPin) * 0.0048828125 * 3;
    delay(10);
  }  
  return voltageSum / NUMBER_OF_SAMPLES;
}

